﻿/**
 * @author Simon Bogutzky (MI03-HS-BREMEN)
 * @version 1.0
 *
 * ItemSelectionBuilder
 *
 * liest XML Dateien und baut aus den Informationen des gelesen XML-Objkets verschiedene Objekte und überreicht sie dem Leveleditor
 */
//XPathAPI für direkte Zugriffe in XML-objebkten über absolute Pfadangaben
//weitere Infos unter : http://download.macromedia.com/pub/documentation/en/flash/fl8/XpathAPI.pdf (Stand 14.01.07)
import mx.xpath.XPathAPI;
import mx.utils.Delegate;
class ItemSelectionBuilder {
	////Objekt Variablen
	private var mParentMovie:MovieClip;
	private var sLevelPath:String;
	private var xmlContent:XML;
	////Konstruktor
	public function ItemSelectionBuilder(par:MovieClip, selectionName:String) {
		this.mParentMovie = par;
		this.sLevelPath = "selection/"+selectionName+".xml";
	}
	////Funktionen
	/**
	 * build
	 * 
	 * liest eine XML-Datei, sucht nach dem angegeben Level und baut daraus einen Pool an Objekten
	 * der an den Leveleditor weitergegeben wird.
	 * dabei wird das empfangene XML-objekt zur Aufbereitung an die "buildElements"-methode zur Aufbereitung delegiert.
	 *
	 */
	public function build():Void {
		xmlContent = new XML();
		xmlContent.ignoreWhite = true;
		xmlContent.load(this.sLevelPath);
		xmlContent.onLoad = Delegate.create(this, buildElements);
	}
	/**
	 * buildElements
	 *
	 * baut Objekte aus einem XML-Objekt und reicht sie dann zum Leveleditor weiter
	 *
	 * @param succes ein von Flash erzeugter Wert, der angiebt, ob eine XML-Objekt erfolgreich geladen wurde oder nicht
	 */
	public function buildElements(success:Boolean):Void {
		if (success && selectionExists(xmlContent.childNodes)) {
			var selectionNodes:Array = xmlContent.childNodes;
			for (var i:Number = 0; i<selectionNodes.length; i++) {
				var elementNodes:Array = selectionNodes[i].childNodes;
				//definiere Floor arrays
				var itemArray:Array = new Array();
				for (var k:Number = 0; k<elementNodes.length; k++) {
					if(elementNodes[k].nodeName=="item")
						{
							var xOrdinate:Number   = Number(getNodeValue(elementNodes[k],"item","x"));
							var yOrdinate:Number   = Number(getNodeValue(elementNodes[k],"item","y"));
							var graphicPath:String = getNodeValue(elementNodes[k],"item","graphicpath");
							var hideout:String     = getNodeValue(elementNodes[k],"item","hideout");
							itemArray.push(new Item(xOrdinate,yOrdinate,graphicPath,hideout));
						}
				}
				//schick alles an den Leveleditor
				this.mParentMovie.initializeSelection(itemArray);
			}
		} else {
			//Fehlerausgabe
			trace("Auswahl nicht gefunden");
			if (!success) {
				//Fehlerausgabe
				trace("Netzwerkfehler aufgetreten");
			}
		}
	}
	/** 
	 * selectionExists
	 * 
	 * überprüft die Selectionnodes auf die Existenz einer Auswahl
	 * 
	 * @param selectionNodes sind die Knoten der Auswahl
	 * @return selectionExistenz wird true wenn eine Auswahl gefunden wurde und false wenn nicht
	 */
	private function selectionExists(selectionNodes:Array):Boolean {
		var selectionExistence:Boolean = false;
		for (var i:Number = 0; i<selectionNodes.length; i++) {
			if (selectionNodes[i].nodeName == "selection") {
				selectionExistence = true;
			}
		}
		return selectionExistence;
	}
	/**
	 * getNodeValue
	 *
	 * greift auf XML-nodes über absolute Pfadangaben zu und gibt deren Werte wieder
	 *
	 * @param xmlData  ist das XML-Objekt, in welchen nach dem Knoten gesucht werden soll
	 * @param nodePath ist der absolute Pfad des Knotens
	 * @param nodeName ist der gesuchte Knotenname
	 * @return String  ist der Wert eines Knotens
	 */
	private function getNodeValue(xmlData:XMLNode, nodePath:String, nodeName:String):String {
		//sucht zum Beipspiel in <floor><nme>floor0<nme><<x>1</x><y>1</y><graphicpath>http://www.terranoxa.de/texture/floor0.jpg</graphicpath></floor>
		//nach dem x-wert wenn nodePath = floor und nodeName = x ist.
		return XPathAPI.selectSingleNode(xmlData, "/"+nodePath+"/"+nodeName).firstChild.nodeValue;
	}
}
