﻿/**
 * @author Robert Brauer (MI05-HS-BREMEN)
 * @version 1.0
 *
 * LevelBuilder
 *
 * liest XML Dateien und baut aus den Informationen des gelesen XML-Objkets verschiedene Objekte und überreicht sie der LevelEngine
 */

//XPathAPI für direkte Zugriffe in XML-objebkten über absolute Pfadangaben
//weitere Infos unter : http://download.macromedia.com/pub/documentation/en/flash/fl8/XpathAPI.pdf (Stand 14.01.07)
import mx.xpath.XPathAPI;
import mx.utils.Delegate;

class LevelBuilder
{	
	////Objekt Variablen
	
	private var mParentMovie:MovieClip;
	private var sLevelPath:String;
	private var nLevel:Number;
	private var xmlContent:XML;
	
	////Konstruktor
	
	public function LevelBuilder(par:MovieClip,gameName:String)
	{
		this.mParentMovie = par;
		//this.sLevelPath   = "http://www.terranoxa.de/level/"+gameName+".xml";
		this.sLevelPath   = "level/"+gameName+".xml";
	}

	////Funktionen
	
	/**
	 * build
	 * 
	 * liest eine XML-Datei, sucht nach dem angegeben Level und baut daraus einen Pool an Objekten
	 * der an die Levelengine weitergegeben wird.
	 * dabei wird das empfangene XML-objekt zur Aufbereitung an die "buildElements"-methode zur Aufbereitung delegiert.
	 *
	 * @param level ist die Nummer des Levels, welches geladen werden soll
	 */
	public function build(level:Number):Void
	{
		nLevel = level;
		xmlContent = new XML();
		xmlContent.ignoreWhite = true;
		xmlContent.load(this.sLevelPath);
		xmlContent.onLoad = Delegate.create(this,buildElements);
	}
	
	/**
	 * buildElements
	 *
	 * baut Objekte aus einem XML-Objekt und reicht sie dann zur Levelengine weiter
	 *
	 * @param succes ein von Flash erzeugter Wert, der angiebt, ob eine XML-Objekt erfolgreich geladen wurde oder nicht
	 */
	public function buildElements(success:Boolean)
	{
		if(success&&levelExists(xmlContent.childNodes,nLevel))
		{
			var levelNodes:Array = xmlContent.childNodes;
			for(var i:Number=0;i<levelNodes.length;i++)
			{
				if(levelNodes[i].nodeName=="level"+nLevel)
				{
					var elementNodes:Array   = levelNodes[i].childNodes;
					
					//definiere Objekte arrays
					var floorArray:Array     = new Array();
					var wallArray:Array      = new Array();
					var doorArray:Array      = new Array();
					var goalArray:Array      = new Array();
					var itemArray:Array      = new Array();
					var opponentArray:Array  = new Array();
					
					for(var k:Number=0;k<elementNodes.length;k++)
					{
						if(elementNodes[k].nodeName=="floor")
						{
							//erstelle ein Floor objekt und häng es ein
							var xOrdinate:Number   = Number(getNodeValue(elementNodes[k],"floor","x"));
							var yOrdinate:Number   = Number(getNodeValue(elementNodes[k],"floor","y"));
							var graphicPath:String = getNodeValue(elementNodes[k],"floor","graphicpath");
							floorArray.push(new Floor(xOrdinate,yOrdinate,graphicPath));
							//trace("boden auf ("+xOrdinate+" "+yOrdinate+")");
						}
						if(elementNodes[k].nodeName=="wall")
						{
							//erstelle ein Wall objekt und häng es ein
							var xOrdinate:Number   = Number(getNodeValue(elementNodes[k],"wall","x"));
							var yOrdinate:Number   = Number(getNodeValue(elementNodes[k],"wall","y"));
							var dir:Number         = Number(getNodeValue(elementNodes[k],"wall","direction"));
							var graphicPath:String = getNodeValue(elementNodes[k],"wall","graphicpath");
							wallArray.push(new Wall(xOrdinate,yOrdinate,graphicPath,dir));
							//trace("wand auf ("+xOrdinate+" "+yOrdinate+") in Richtung "+dir);
						}
						if(elementNodes[k].nodeName=="door")
						{
							//erstelle ein Door objekt und häng es ein
							var xOrdinate:Number        = Number(getNodeValue(elementNodes[k],"door","x"));
							var yOrdinate:Number        = Number(getNodeValue(elementNodes[k],"door","y"));
							var dir:Number              = Number(getNodeValue(elementNodes[k],"door","direction"));
							var frameGraphicPath:String = getNodeValue(elementNodes[k],"door","graphicpath");
							var doorGraphicPath:String  = getNodeValue(elementNodes[k],"door","graphicpath1");
							var requirement:String      = getNodeValue(elementNodes[k],"door","requirement");
							doorArray.push(new Door(xOrdinate,yOrdinate,frameGraphicPath,doorGraphicPath,dir,requirement));
							//trace("tür auf ("+xOrdinate+" "+yOrdinate+") in Richtung "+dir+" "+requirement);
						}
						if(elementNodes[k].nodeName=="goal")
						{
							//erstelle ein Goal objekt und häng es ein
							var xOrdinate:Number   = Number(getNodeValue(elementNodes[k],"goal","x"));
							var yOrdinate:Number   = Number(getNodeValue(elementNodes[k],"goal","y"));
							var graphicPath:String = getNodeValue(elementNodes[k],"goal","graphicpath");
							goalArray.push(new Goal(xOrdinate,yOrdinate,graphicPath));
							//trace("Ziel auf ("+xOrdinate+" "+yOrdinate+")");
						}
						if(elementNodes[k].nodeName=="item")
						{
							//erstelle ein Goal objekt und häng es ein
							var xOrdinate:Number   = Number(getNodeValue(elementNodes[k],"item","x"));
							var yOrdinate:Number   = Number(getNodeValue(elementNodes[k],"item","y"));
							var graphicPath:String = getNodeValue(elementNodes[k],"item","graphicpath");
							var hideout:String     = getNodeValue(elementNodes[k],"item","hideout");
							itemArray.push(new Item(xOrdinate,yOrdinate,graphicPath,hideout));
							//trace("Item auf ("+xOrdinate+" "+yOrdinate+")"+hideout);
						}
						if(elementNodes[k].nodeName=="opponent")
						{
							//erstelle ein Goal objekt und häng es ein
							var xOrdinate:Number    = Number(getNodeValue(elementNodes[k],"opponent","x"));
							var yOrdinate:Number    = Number(getNodeValue(elementNodes[k],"opponent","y"));
							var dir:Number          = Number(getNodeValue(elementNodes[k],"opponent","direction"));
							var persecution:Boolean = getNodeValue(elementNodes[k],"opponent","persecution")=="true"?true:false;
							opponentArray.push(new Opponent(xOrdinate,yOrdinate,dir,persecution));
							//trace("Gegner auf ("+xOrdinate+" "+yOrdinate+") in Richtung "+dir);
						}
						// mit meheren ifs können hier mehere Objekte erstellt werden
						//sofern sie wie die floors definiert sind
					}
					
					//schick alles an den Manager
					//trace("level "+nLevel+" is fertig");
					this.mParentMovie.initializeLevel(floorArray,wallArray,doorArray,goalArray,itemArray,opponentArray);
				}
			}
		}
		else
		{
			//wenn das Level nicht gefunden wurde, dann wird das spiel beendet auch wenn es sich hierbei um einen Netzwerkfehler handeln sollte
			this.mParentMovie.gameFinished();
			if(!success)
			{
				//hier könnten man die popups des parents benutzen
				_parent.showInfo("Netzwerkfehler","aufgetreten");
			}
		}
	}
	
	/** 
	 * levelExists
	 * 
	 * überprüft die Levelnodes auf die Existenz eines Levels
	 * 
	 * @param levelNodes sind die Knoten der Level
	 * @param nLevel ist die Zahl des gesuchten Levels
	 * @return levelExistenz wird true wenn ein Level gefunden wurde und false wenn nicht
	 */
	private function levelExists(levelNodes:Array,nLevel:Number):Boolean
	{
		var levelExistence:Boolean = false;
		
		for(var i:Number=0;i<levelNodes.length;i++)
		{
			if(levelNodes[i].nodeName=="level"+nLevel)
			{
				levelExistence = true;
			}
		}
		
		return levelExistence;
	}
	
	/**
	 * getNodeValue
	 *
	 * greift auf XML-nodes über absolute Pfadangaben zu und gibt deren Werte wieder
	 *
	 * @param xmlData  ist das XML-Objekt, in welchen nach dem Knoten gesucht werden soll
	 * @param nodePath ist der absolute Pfad des Knotens
	 * @param nodeName ist der gesuchte Knotenname
	 * @return String  ist der Wert eines Knotens
	 */
	private function getNodeValue(xmlData:XMLNode,nodePath:String,nodeName:String):String
	{
		//sucht zum Beipspiel in <floor><x>1</x><y>1</y><graphicpath>http://www.terranoxa.de/texture/floor0.jpg</graphicpath></floor>
		//nach dem x-wert wenn nodePath = floor und nodeName = x ist.
		return XPathAPI.selectSingleNode(xmlData,"/"+nodePath+"/"+nodeName).firstChild.nodeValue;
	}
}
